unit sCustomMenuManager;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.3, 10-03-2002
*************************************************************************)
{DEFINE SINGLE}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, sConst,
  Menus, sUtils, ExtCtrls, sGraphUtils, sStylePassive;

type
  TsCustomMenuManager = class(TComponent)
  private
    { Private declarations }
    FsStyle : TsPassivePaintStyle;
    FMnuList : TList;
    FActive : boolean;
    FLeftLineWidth : integer;
    FFont : TFont;
    procedure SetActive(const Value: boolean);
    procedure SetFont(const Value: TFont);
  protected
    { Protected declarations }
//    procedure WndProc (var Message: TMessage); override;
    procedure sMeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
    procedure sAdvancedDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
  public
    FForm: TCustomForm;
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate;
    procedure InitMenus(A: boolean); 
    procedure HookMenu(MainMenu: TMainMenu; FActive: boolean);
  published
    { Published declarations }
    property sStyle : TsPassivePaintStyle read FsStyle write FsStyle;
    property Active: boolean read FActive write SetActive;
    property Font : TFont read FFont write SetFont;
  end;

  TsMenuManager = class(TsCustomMenuManager)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
  published
    { Published declarations }
  end;

{$IFNDEF SINGLE}
{$R *.DCR}
procedure Register;
{$ENDIF}

implementation


{ TsCustomMenuManager }

function GlyphSize(Item: TMenuItem; Top: boolean): TSize;
begin
  Result.cx := 0;
  Result.cy := 0;
  if Top then begin
    if Assigned(Item.GetParentMenu.Images) then begin
      Result.cx := 28 * integer(Item.ImageIndex >= 0);
      Result.cy := 28 * integer(Item.ImageIndex >= 0);
    end
    else begin
      Result.cx := 0;
      Result.cy := 0;
    end;
  end
  else begin
    if Assigned(Item.GetParentMenu.Images) then begin
      Result.cx := Item.GetParentMenu.Images.Width;
      Result.cy := Item.GetParentMenu.Images.Height;
    end
    else begin
      Result.cx := 16;
      Result.cy := 16;
    end;
  end;
end;

constructor TsCustomMenuManager.Create(AOwner: TComponent);
begin
  inherited;
  FsStyle := TsPassivePaintStyle.Create(Self);
  FMnuList := TList.Create;
  FForm := GetParentForm(TControl(AOwner));
  FFont := TFont.Create;
  
  FLeftLineWidth := 0;
  sStyle.COC := COC_TsCustomMenuManager;
end;

procedure TsCustomMenuManager.sAdvancedDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
var
  P : TPoint;
  R, gRect, cRect : TRect;
  i: integer;
  Item : TMenuItem;
  DontPaintBG: boolean;
  Text: string;
  function TextRect: TRect; begin
    Result := aRect;
    inc(Result.Left, 6 + GlyphSize(Item, DontPaintBG).cx);
    if (Item.Bitmap <> nil) and (Item.Bitmap.Width > 0) then begin
      inc(Result.Left, Item.Bitmap.Width);
    end;
  end;
begin
  Item := TMenuItem(Sender);

  if TMenuItem(Sender).Caption = '-' then begin
//    gRect := Rect(0, 0, WidthOf(ARect), Item.Parent.Count * HeightOf(aRect));
//    sStyle.PaintBG(gRect);
    i := 1;
    r :=  Rect(aRect.Left + 3,
               aRect.Top,
               aRect.Right - 3,
               aRect.Bottom);
    DrawRectangleOnDC(sStyle.FCacheBmp.Canvas.Handle,
                      r,
                      sStyle.Painting.ColorBorderBottom,
                      sStyle.Painting.ColorBorderTop,
                      i);
    BitBlt(ACanvas.Handle, aRect.Left, aRect.Top, WidthOf(aRect), HeightOf(aRect),
            sStyle.FCacheBmp.Canvas.Handle, aRect.Left, aRect.Top, SrcCopy);
    Exit;
  end;

  DontPaintBG := False;
  if Item.Parent.GetParentMenu is TMainMenu then begin
    for i := 0 to Item.GetParentMenu.Items.Count - 1 do begin
      if Item.GetParentMenu.Items[i] = Item then begin
        DontPaintBG := True;
        break;
      end;
    end;
  end;

  if DontPaintBG then begin
    gRect := Rect(0, 0, FForm.Width, ARect.Bottom + 4);
  end
  else begin
    gRect := Rect(0, 0, WidthOf(ARect), Item.Parent.Count * HeightOf(aRect));
  end;

  sStyle.FCacheBmp.Width := WidthOf(gRect);
  sStyle.FCacheBmp.Height := HeightOf(gRect);

  if DontPaintBG then begin
    sStyle.FCacheBmp.Canvas.Brush.Color := clMenu;
    sStyle.FCacheBmp.Canvas.FillRect(gRect);
  end
  else begin
    sStyle.PaintBG(gRect);
    sStyle.PaintBorder(sStyle.FCacheBmp.Canvas.Handle, gRect, False, 0);
  end;

  R := ARect;
  dec(R.Right, 1);
  dec(R.Bottom, 1);
  if (odSelected in State) then begin
    if (msColor in sStyle.Selection.Mode) then begin
      FadeRect(sStyle.FCacheBmp.Canvas, R, sStyle.FCacheBmp.Canvas, Point(R.Left, R.Top),
                sStyle.Painting.Transparency, sStyle.Painting.ColorActive, 0, ssRectangle);
    end;
    sStyle.PaintBorder(sStyle.FCacheBmp.Canvas.Handle, R,
                       not(DontPaintBG) or (sStyle.Painting.Bevel = bvNone),
                       integer(DontPaintBG));
  end;


  if odChecked in State then begin
    cRect.Top    := R.Top + (HeightOf(R) - GlyphSize(Item, DontPaintBG).cy) div 2;
    cRect.Left   := R.Left + cRect.Top - R.Top;
    cRect.Right  := cRect.Left + GlyphSize(Item, DontPaintBG).cx;
    cRect.Bottom := cRect.Top + GlyphSize(Item, DontPaintBG).cy;
    FadeRect(sStyle.FCacheBmp.Canvas,
             cRect,
             sStyle.FCacheBmp.Canvas,
             Point(cRect.Left, cRect.Top),
             50,
             clWhite, 0, ssRectangle);
    i := 1;
    DrawRectangleOnDC(sStyle.FCacheBmp.Canvas.Handle,
                      cRect,
                      sStyle.Painting.ColorBorderBottom,
                      sStyle.Painting.ColorBorderTop,
                      i);
//    sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
//    sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
    PaintCheck(sStyle.FCacheBmp.Canvas, cRect, Item.Enabled, clBlack);
  end;
  
  if Assigned(Item.GetParentMenu.Images) and (Item.ImageIndex >= 0) then begin
    try
    Item.GetParentMenu.Images.Draw(sStyle.FCacheBmp.Canvas,
                                   aRect.Left, aRect.Top,
                                   Item.ImageIndex,
                                   not (odDisabled in State));
    except alert;
    end;
  end;

  if odDefault in State then begin
    sStyle.FCacheBmp.Canvas.Font.Style := sStyle.FCacheBmp.Canvas.Font.Style + [fsBold];
  end
  else begin
    sStyle.FCacheBmp.Canvas.Font.Style := sStyle.FCacheBmp.Canvas.Font.Style - [fsBold];
  end;
  sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
  R := TextRect;


  if Assigned(FFont) then
    sStyle.FCacheBmp.Canvas.Font.Assign(FFont);
  sGraphUtils.WriteText(sStyle.FCacheBmp.Canvas, PChar(Item.Caption), Item.Enabled, R, DT_VCENTER);
  Text := ShortCutToText(TMenuItem(Sender).ShortCut);
  r := aRect;
  dec(r.Right, 8);
  sGraphUtils.WriteText(sStyle.FCacheBmp.Canvas, PChar(Text), Item.Enabled, R, DT_VCENTER or DT_RIGHT);

  BitBlt(ACanvas.Handle, aRect.Left, aRect.Top, WidthOf(aRect), HeightOf(aRect),
            sStyle.FCacheBmp.Canvas.Handle, aRect.Left, aRect.Top, SrcCopy);
end;

procedure TsCustomMenuManager.Invalidate;
begin
end;

procedure TsCustomMenuManager.InitMenus(A: boolean);
var
  i, j : integer;
begin
  FMnuList.Clear;
  for i := 0 to FForm.ComponentCount - 1 do begin
    if (FForm.Components[i] is TMainMenu) or (FForm.Components[i] is TPopupMenu) then begin
      FMnuList.Add(TMainMenu(FForm.Components[i]));
      HookMenu(TMainMenu(FForm.Components[i]), A);
    end
  end;
end;

procedure TsCustomMenuManager.SetActive(const Value: boolean);
var
  i : integer;
begin
  if FActive <> Value then begin
    InitMenus(Value);
    FActive := Value;
  end;
end;

procedure TsCustomMenuManager.HookMenu(MainMenu: TMainMenu; FActive: boolean);
var
  i : integer;
  procedure HookSubItems(Item: TMenuItem);
  var
    i : integer;
  begin
    for i := 0 to Item.Count - 1 do begin
      if FActive then begin
        Item.Items[i].OnAdvancedDrawItem := sAdvancedDrawItem;
        Item.Items[i].OnMeasureItem := sMeasureItem;
      end
      else begin
        if addr(Item.Items[i].OnAdvancedDrawItem) = addr(TsCustomMenuManager.sAdvancedDrawItem) then
          Item.Items[i].OnAdvancedDrawItem := nil;
        if addr(Item.Items[i].OnMeasureItem) = addr(TsCustomMenuManager.sMeasureItem) then
          Item.Items[i].OnMeasureItem := nil;
      end;
      HookSubItems(Item.Items[i]);
    end;
  end;
begin
  if MainMenu.Items = nil then Exit;
  for i := 0 to MainMenu.Items.Count - 1 do begin
    if FActive then begin
      MainMenu.Items[i].OnAdvancedDrawItem := sAdvancedDrawItem;
      MainMenu.Items[i].OnMeasureItem := sMeasureItem;
    end
    else begin
      if addr(MainMenu.Items[i].OnAdvancedDrawItem) = addr(TsCustomMenuManager.sAdvancedDrawItem) then
        MainMenu.Items[i].OnAdvancedDrawItem := nil;
      if addr(MainMenu.Items[i].OnMeasureItem) = addr(TsCustomMenuManager.sMeasureItem) then
        MainMenu.Items[i].OnMeasureItem := nil;
    end;
    HookSubItems(MainMenu.Items[i]);
  end;
  MainMenu.OwnerDraw := FActive;
end;

procedure TsCustomMenuManager.sMeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
var
  Text: string;
  Divider: boolean;
  T: boolean;
  Item: TMenuItem;
  i: integer;
begin
  Item := TMenuItem(Sender);
  t := False;
  if Item.Parent.GetParentMenu is TMainMenu then begin
    for i := 0 to Item.GetParentMenu.Items.Count - 1 do begin
      if Item.GetParentMenu.Items[i] = Item then begin
        t := True;
        break;
      end;
    end;
  end;

  Divider := TMenuItem(Sender).Caption = '-';

  if Divider then begin
    Text := '';
  end
  else begin
    Text := TMenuItem(Sender).Caption;
    Text := Text + iff(ShortCutToText(TMenuItem(Sender).ShortCut) = '', '', '__' + ShortCutToText(TMenuItem(Sender).ShortCut));
  end;

  if Assigned(FFont) then
    ACanvas.Font.Assign(FFont);
  Width := Maxi(Width, ACanvas.TextWidth(Text) + GlyphSize(Item, t).cx);

  if Divider then
    Height := 2
  else begin
    Height := Round(ACanvas.TextHeight('W') * 1.75);
  end;

  inc(Width, FLeftLineWidth * integer(not t));
end;

destructor TsCustomMenuManager.Destroy;
begin
  Active := False;
  FreeAndNil(FsStyle);
  FreeAndNil(FMnuList);
  FreeAndNil(FFont);
  inherited;
end;

{ TsMainMenuManager }

constructor TsMenuManager.Create(AOwner: TComponent);
begin
  inherited;
  sStyle.COC := COC_TsMenuManager;
end;

procedure TsCustomMenuManager.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
end;

{$IFNDEF SINGLE}
procedure Register;
begin
  RegisterComponents('sTools', [TsMenuManager]);
end;
{$ENDIF}

end.
